#pragma rtGlobals=3		// Use modern global access method and strict wave access.

//This procedure contains functions to load spr
//data files.

Function/S LoadDataFile([DataF,EventF,expN])	//SPR Data File loader
	String DataF,EventF,expN
	
		
	DFREF saveDFR = GetDataFolderDFR()		// Save current DF for restore
	
	SetDataFolder root:
	
	String wexplist="ExperimentList"
	
	If (!waveexists($wexplist))
		make/T/N=(1,6) $wexplist			//Holds Information of the loaded Experiments
		SetDimLabel 1,0,Folder,$wexplist
		SetDimLabel 1,1,DCRate,$wexplist
		SetDimLabel 1,2,Name,$wexplist
	else
		Redimension/N=(dimsize($wexplist,0)+1,-1) $wexplist
	endif
	
	Wave/T wexp=$wexplist
	
	String DataFile=""
	String EventsFile=""
	//abfragen der pfade
	If(!ParamIsDefault(DataF) && !ParamIsDefault(EventF))
		DataFile=DataF
		EventsFile=EventF
	else
		DataFile=DoOpenMultiFileDialog()
		EventsFile=DoOpenMultiFileDialogEvents()
	EndIf
	//abfragen des experiment namens
	String ExpName
	If(!ParamIsDefault(expN))
		ExpName=expN
	else
		Prompt ExpName, "Experiment Folder Name (e.g Experiment Date)"
		DoPrompt "Give Foldername...",ExpName
		if (V_Flag)
			Print "Need Foldername"
			return ""	// User canceled
		endif
	EndIf

	//folder machen
	String sNewDataFolder="root:E"+ExpName
	NewDataFolder/O/S $sNewDataFolder	//Create new Data Folder in root and make it active
	
	Variable/G DCRate	//Contains datacapture rate
	
	Make/T/N=10 wDataRef
	//dateien laden
	//Progress Panel
	getwindow kwframeinner wsize
	NewPanel/FLT /N=Loading/W=(V_right/2,V_bottom/2,V_right/2+320,V_bottom/2+40)
	ValDisplay valdisp0,pos={10,10},size={300,20}
	ValDisplay valdisp0,limits={0,100,0},barmisc={0,0}
	ValDisplay valdisp0,value= _NUM:0
	ValDisplay valdisp0,mode=4 // candy stripe
	ValDisplay valdisp0,highColor=(65535,0,0)
	SetActiveSubwindow _endfloat_
	DoUpdate/W=Loading/E=1 // mark this as our progress window
	SetWindow Loading,hook(spinner)= MySpinner
	
	String s
	If(!stringmatch(DataFile,""))
		s=LoadDataWaves(DataFile)	//Load Data Waves
		DCRate=CheckDCRate($stringfromlist(1,s))
		wDataRef[0]=Stringfromlist(1,s)	//Time trace
		wDataRef[1]=Stringfromlist(2,s)	//Flow cell 1
		wDataRef[2]=Stringfromlist(3,s)	//Flow cell 2
		wDataRef[3]=Stringfromlist(4,s)	//Flow cell 3
		wDataRef[4]=Stringfromlist(5,s)	//Flow cell 4
		wexp[(dimsize(wexp,0)-1)][%Folder]=ExpName
		wexp[(dimsize(wexp,0)-1)][%Name]=DataFile
		wexp[(dimsize(wexp,0)-1)][%DCRate]=num2str(DCRate)
	else
		print "No valid Path to Data File found or canceled by User!"
		return ""
	endif
	
	If(!stringmatch(EventsFile,""))
		s=LoadEventsWave(EventsFile)	//Open Events File
		wDataRef[5]=Stringfromlist(2,s)	//Event Names
		wDataRef[6]=Stringfromlist(1,s)	//Event Times
	else
		print "No Events File Loaded"
	endif
	
//	SetDataFolder saveDFR	//restore DF
	
	KillWindow Loading
	
	return sNewDataFolder
End

Function LoadSamplesFile(num,[pathtoFolder,pathtoFile])	//Load Sample Names and Concentration extracted from SPR-Logfile
	String pathtoFolder,pathtoFile
	Variable num //if 0: read conc from file; if 1: read conc from file but only take first (highest conc) and calc dilutions.
	
	String Folder=""
	//asks for the experiment number for which you want to load the samples list file
	String newDF
	If(!ParamIsDefault(pathtoFolder))
		newDF=pathtoFolder
	else
		newDF=ChooseActiveDF(0)
	EndIf
	
	DFREF saveDFR = GetDataFolderDFR()
	
	SetDataFolder newDF
	String sPathtoSample=""
	If(!ParamIsDefault(pathtoFile))
		sPathtoSample=pathtoFile
	else
		sPathtoSample=DoOpenMultiFileDialogSamples()
	EndIf
	
	String LoadedWaves=LoadSamplesWave(sPathtoSample)
	
	String concs=scanforconcs($(stringfromlist(2,LoadedWaves)),num)
	
//	edit/K=1/W=(0,0,498.75,658.25) $(stringfromlist(1,LoadedWaves)); appendtotable $(stringfromlist(2,LoadedWaves)); appendtotable $concs

	SetDataFolder saveDFR	
End



///////////////////////////////////////////
///////////////////////////////////////////

Function MySpinner(s)

	STRUCT WMWinHookStruct &s

	if( s.eventCode == 23 )
		ValDisplay valdisp0,value= _NUM:1,win=$s.winName
		DoUpdate/W=$s.winName
		if( V_Flag == 2 ) // we only have one button and that means abort
			KillWindow $s.winName
			return 1
		endif
	endif
	return 0
End

Function/S DoOpenMultiFileDialog()
	Variable refNum

	String message = "Select an SPR Data file"

	String outputPath
	String fileFilters = "Data Files (*.txt,*.dat,*.csv):.txt,.dat,.csv;"
	fileFilters += "All Files:.*;"
	Open /D /R /F=fileFilters /M=message refNum
	outputPath = S_fileName
	return outputPath		// Will be empty if user canceled
End

Function/S DoOpenMultiFileDialogEvents()
	Variable refNum

	String message = "Select an SPR EVENTS file"

	String outputPath
	String fileFilters = "Data Files (*.txt,*.dat,*.csv):.txt,.dat,.csv;"
	fileFilters += "All Files:.*;"

	Open /D /R /F=fileFilters /M=message refNum
	outputPath = S_fileName
	return outputPath		// Will be empty if user canceled
End

Function/S LoadDataWaves(path) 
	String path

	String baseName = ""

	Splitstring /E=".+\:(.+)\.txt$" path, baseName

//	baseName = ReplaceString("-",baseName,"")
	baseName = ""

	String ColumnInfoStr = ""

	ColumnInfoStr += "C=1,N='SPR"+baseName+"zeit';"
	ColumnInfoStr += "C=1,N='SPR"+baseName+"FC1';"
	ColumnInfoStr += "C=1,N='_skip_';"
	ColumnInfoStr += "C=1,N='SPR"+baseName+"FC2';"
	ColumnInfoStr += "C=1,N='_skip_';"
	ColumnInfoStr += "C=1,N='SPR"+baseName+"FC3';"
	ColumnInfoStr += "C=1,N='_skip_';"
	ColumnInfoStr += "C=1,N='SPR"+baseName+"FC4';"

	LoadWave /Q /O /A /G /D /B=ColumnInfoStr path

	return baseName+";"+S_waveNames

End

Function/S LoadEventsWave(path)
	String path
	
	String baseName=""
	
	Splitstring /E=".+\:(.+)\.txt$" path, baseName
	
	String columninfo = "C=1,F=0,N='eventT';C=2,F=-2,N='eventN';"
	
	LoadWave/Q/A/O/J/D/B=columninfo/V={"\t"," $",0,0}/L={0,0,0,0,2} path

	return baseName+";"+S_waveNames
	
End

Function/S DoOpenMultiFileDialogSamples()
	Variable refNum

	String message = "Select an SPR Samples file"

	String outputPath
	String fileFilters = "Data Files (*.txt,*.dat,*.csv):.txt,.dat,.csv;"
	fileFilters += "All Files:.*;"

	Open /D /R /F=fileFilters /M=message refNum
	outputPath = S_fileName
	return outputPath		// Will be empty if user canceled
End

Function/S LoadSamplesWave(path)
	String path
	
	String baseName=""
	
	Splitstring /E=".+\:(.+)\.txt$" path, baseName
	
	String columninfo = "C=1,F=-2,N='SampleNum';C=1,F=-2,N='SampleName';"
	
	LoadWave/Q/A/O/J/D/B=columninfo/V={"\t"," $",0,0}/L={0,0,0,0,2} path
	
	return baseName+";"+S_waveNames
	
End

Function/S scanforconcs(w,num)
	Wave/T w
	Variable num //if 0: read conc from file; if 1: read conc from file but only take first (highest conc) and calc dilutions.

	String wName="Concs"
	
	Make/D/O/N=0 $wName
	
	Wave wC=$wName
	
	variable numr
	string t1,t2
	
	Variable i,j=0
	For(i=0;i<numpnts(w);i+=1)
		string temp=w[i]
		sscanf	temp, "%s %f%s",t1,numr,t2
		If(numr)
			redimension/N=(numpnts(wC)+1) wC
			if(stringmatch(t2,"mM"))
			wC[j]=numr*1e-3
			elseif(stringmatch(t2,"uM"))
			wC[j]=numr*1e-6
			else
			wC[j]=numr*1e-9
			endif
			j+=1
		EndIf
	endfor
	sort wC, wC

	if(num)
		For(i=0;i<numpnts(wC)-1;i+=1)
			wC[i]=wC[numpnts(wC)-1]*2^(-(numpnts(wC)-1-i))

		EndFor
	endif

	
	return wName
End

Function CheckDCRate(w)
Wave w

return 1/(w[2]-w[1])
End