#pragma rtGlobals=3		// Use modern global access method and strict wave access.


//Creates Wave with Start and End Times of Injections.
//It takes the value from the log file and calculates the exact by a
//search algorithm.
//Output is a wave reference to the wave holding all the times (points)
Function GetTimes()

	Variable/G DCRate	//Data capture rate in Hz.
	Variable/G gRef		//Reference Cell Number
	
	Variable/G gToffset
	Variable/G gPoffset
	
	if(exists("gToffset"))
	else
		gToffset=0
		gPoffset=0
	endif
	
	Wave/T wDataRef	//Contains names of data waves
	Wave/T wName = $(wDataRef[5])	//Contains Name of event (from logfile)
	Wave wTime = $(wDataRef[6])	//Contains Time of event (from logfile)
	Wave wd1 = $(wDataRef[7])+"SS"		//Data Wave 
	Wave wd2 = $(wDataRef[8])+"SS"		//Data Wave
	Wave wd3 = $(wDataRef[9])+"SS"		//Data Wave
	
	If(!waveexists(wd1))
		print "Prepare Data first!"
		return -1
	Endif
	
	CleanUp(wTime,wName) //Clean Up name wave and time wave (removes "Compressor: ON  and OFF lines".
	
	make/D/O/N=(0,9) Times	//Contains all time points for association and dissociation of 
								//single injections.
								//StartAssLog;StartDissLog;EndDissLog;
								//StartAssCalc for each FC (3);StartDissCalc for each FC (3);

	
	Variable i,j,offset=0,count=0
	//Exctract Association Start
	For(i=1;i<(numpnts(wName)-1);i+=1)
		If(stringmatch(wname(i),"Flow path:*"))
			offset=count*gToffset
			count+=1
		EndIf
		If(stringmatch(wName(i), "Inject: Start*") & stringmatch(wName(i+1), "Inject: Ready*") & !stringmatch(wName(i-1),"Inject sequence: Start*") & !stringmatch(wName(i-1), "Inject: Ready*"))
			Redimension /N=(dimsize(Times,0)+1,-1) Times
			Times[dimsize(Times,0)-1][0]=wTime[i]//+offset
		Endif		
	EndFor
	
	//Extract Dissociation Start and Stop
	j=0;offset=0;count=0
	For(i=1;i<(numpnts(wName)-2);i+=1)
		If(stringmatch(wname(i),"Flow path:*"))
			offset=count*gToffset
			count+=1
		EndIf
		If(stringmatch(wName(i), "Inject: Start*") & stringmatch(wName(i+1), "Inject: Ready*") & stringmatch(wName(i+2), "Dissociation*") & !stringmatch(wName(i-1),"Inject sequence: Start*") & !stringmatch(wName(i-1), "Inject: Ready*"))
			Times[j][1]=wTime[i+1]//+offset	//Start
			Times[j][2]=wTime[i+2]//+offset	//Stop
			j+=1
		Endif		
	EndFor
	
	//Correct Association Start Points for each flow cell
	//Progress Panel
	NewPanel /N=ProgressPanel /W=(285,111,739,193)
	ValDisplay valdisp0,pos={18,32},size={342,18},limits={0,(dimsize(Times,0)),0},barmisc={0,0}
	ValDisplay valdisp0,value= _NUM:0
	ValDisplay valdisp0,mode= 3
	ValDisplay valdisp0,highColor=(0,65535,0)
	Button bStop,pos={375,32},size={50,20},title="Stop"
	DoUpdate /W=ProgressPanel /E=1	// mark this as our progress window

	For(i=0;i<(dimsize(Times,0));i+=1)
		Times[i][3] = CorrectAssStartPoint(wd1,(Times[i][0]))/(dcrate*10)
		Times[i][4] = CorrectAssStartPoint(wd2,(Times[i][0]))/(dcrate*10)
		Times[i][5] = CorrectAssStartPoint(wd3,(Times[i][0]))/(dcrate*10)
		ValDisplay valdisp0,value= _NUM:i+1,win=ProgressPanel
		DoUpdate /W=ProgressPanel
		if( V_Flag == 2 )	// we only have one button and that means stop
			break
		endif
	EndFor
	
	KillWindow ProgressPanel
	
	Make/D/FREE/N=(dimsize(Times,0)) diff, diff_hist
	
	Variable d
	For(i=0;i<3;i+=1)	//Corrects calculated start point
	
		diff[]=Times[p][i+3]-Times[p][0]	//Calculates all corrections from before
		Histogram/B={0,0.2,500} diff,diff_Hist
		Wavestats/Q diff_hist
		d=V_maxloc
		For(j=0;j<(dimsize(Times,0));j+=1)
			If((Times[j][i+3]-Times[j][0])>(d*10))	//If correction differs more than
				Times[j][i+3]=Times[j][0]+(d)			//250% of average correction*2
			EndIf											//it gets overwritten.
		EndFor
	EndFor
	
	//Correct Dissociation Start Points for each flow cell
	//Progress Panel
	NewPanel /N=ProgressPanel /W=(285,111,739,193)
	ValDisplay valdisp0,pos={18,32},size={342,18},limits={0,(dimsize(Times,0)),0},barmisc={0,0}
	ValDisplay valdisp0,value= _NUM:0
	ValDisplay valdisp0,mode= 3
	ValDisplay valdisp0,highColor=(0,65535,0)
	Button bStop,pos={375,32},size={50,20},title="Stop"
	DoUpdate /W=ProgressPanel /E=1	// mark this as our progress window

	For(i=0;i<(dimsize(Times,0));i+=1)
		Times[i][6] = CorrectDissStartPoint(wd1,(Times[i][1]))/(dcrate*10)
		Times[i][7] = CorrectDissStartPoint(wd2,(Times[i][1]))/(dcrate*10)
		Times[i][8] = CorrectDissStartPoint(wd3,(Times[i][1]))/(dcrate*10)
		ValDisplay valdisp0,value= _NUM:i+1,win=ProgressPanel
		DoUpdate /W=ProgressPanel
		if( V_Flag == 2 )	// we only have one button and that means stop
			break
		endif
	EndFor
	
	KillWindow ProgressPanel
	
	Make/D/FREE/N=(dimsize(Times,0)) diff, diff_hist
	

	For(i=0;i<3;i+=1)	//Corrects calculated start point
	
		diff[]=Times[p][i+6]-Times[p][1]	//Calculates all corrections from before
		Histogram/B={0,0.05,500} diff,diff_Hist
		Wavestats/Q diff_hist
		d=V_maxloc
		For(j=0;j<(dimsize(Times,0));j+=1)
			If((Times[j][i+6]-Times[j][1])>(d*10))	//If correction differs more than
				Times[j][i+6]=Times[j][1]+d		//250% of average correction*2
			EndIf											//it gets overwritten.
		EndFor
	EndFor
	
	
	///////////////////////Maybe not usable/////////////////////////
	String FolderNames=""
	String nwd1,nwd2,nwd3,nwd4
	splitstring/E="(.{7}$)" (wDataRef[1]+"Diff")
	nwd1=S_value
	splitstring/E="(.{7}$)" (wDataRef[2]+"Diff")
	nwd2=S_value
	splitstring/E="(.{7}$)" (wDataRef[3]+"Diff")
	nwd3=S_value
	splitstring/E="(.{7}$)" (wDataRef[4]+"Diff")
	nwd4=S_value
	
	If(gRef==1)
		FolderNames=nwd2+";"+nwd3+";"+nwd4
	elseif(gRef==2)
		FolderNames=nwd1+";"+nwd3+";"+nwd4
	elseif(gRef==3)
		FolderNames=nwd1+";"+nwd2+";"+nwd4
	elseif(gRef==4)
		FolderNames=nwd1+";"+nwd2+";"+nwd3
	EndIf
	
	//create folders
	NewDataFolder/O $stringfromlist(0,FolderNames)
	NewDataFolder/O $stringfromlist(1,FolderNames)
	NewDataFolder/O $stringfromlist(2,FolderNames)
	String f1,f2,f3
	
	String wr1,wr2,wr3
	wr1=":"+stringfromlist(0,FolderNames)+":wRef"
	wr2=":"+stringfromlist(1,FolderNames)+":wRef"
	wr3=":"+stringfromlist(2,FolderNames)+":wRef"
	Make/D/O/T/N=0 $wr1,$wr2,$wr3
	Wave/T wref1=$wr1
	Wave/T wref2=$wr2
	Wave/T wref3=$wr3
		
	//Copy Injections into created folders
	For(i=0;i<(dimsize(Times,0));i+=1)
		f1=":"+stringfromlist(0,FolderNames)+":"+wDataRef[7]+"I"+num2str(i+1)
		f2=":"+stringfromlist(1,FolderNames)+":"+wDataRef[8]+"I"+num2str(i+1)
		f3=":"+stringfromlist(2,FolderNames)+":"+wDataRef[9]+"I"+num2str(i+1)
		
		Duplicate/O/R=[(Times[i][3])*dcrate*10,Times[i][2]*dcrate*10] wd1 $f1
		Setscale/I x,0,(Times[i][2]-Times[i][3]),"s",$f1
		Redimension /N=(numpnts(wref1)+1) wref1
		wref1[numpnts(wref1)-1]=wDataRef[7]+"I"+num2str(i+1)
		Duplicate/O/R=[(Times[i][4])*dcrate*10,Times[i][2]*dcrate*10] wd2 $f2
		Setscale/I x,0,(Times[i][2]-Times[i][4]),"s",$f2
		Redimension /N=(numpnts(wref2)+1) wref2
		wref2[numpnts(wref2)-1]=wDataRef[8]+"I"+num2str(i+1)
		Duplicate/O/R=[(Times[i][5])*dcrate*10,Times[i][2]*dcrate*10] wd3 $f3
		Setscale/I x,0,(Times[i][2]-Times[i][5]),"s",$f3
		Redimension /N=(numpnts(wref3)+1) wref3
		wref3[numpnts(wref3)-1]=wDataRef[9]+"I"+num2str(i+1)
		
	EndFor
	///////////////////////Maybe not usable end/////////////////////////
End

//Algorithm to find the exact start point of the Association part.
Function CorrectAssStartPoint(w,num)
	Wave w	//Data Wave where to search for start point
	Variable num //Association Start time from Log.
	
	Variable/G dcRate
	
	Variable noise=CalcNoise(w)+2	//w is interpolated data wave!
	Variable ps	//actual position of searcher
	Variable p0=(num-2)*dcrate*10+1	//start point
	Variable p1=p0-6*dcRate*10+1	//go back in time by 6s
	Variable p2=p1-10*dcRate*10+1	//go back in time by 10s relative to p1
	//linefit for baseline between p2 and p1.
	CurveFit/Q line, w[p2,p1]
	Wave W_coef
	make/FREE/D/N=(numpnts(w)) wt=w
	setscale/I x,(Leftx(w)),(rightx(w)), wt
	wt[]=wt[p]-(W_coef[0]+W_coef[1]*(p/dcrate/10))
	ps=p0
	Do
		ps+=1
	While(wt[ps]<=noise && ps<(numpnts(wt)-2))
	
	return ps
	
End
//Algorithm to find the exact start point of the Dissociation part.
Function CorrectDissStartPoint(w,num)
	Wave w	//Data Wave where to search for start point
	Variable num //Association Start time from Log.
	
	Variable/G dcRate
	
	Variable noise=CalcNoise(w)+2	//w is interpolated data wave!
	Variable ps	//actual position of searcher
	Variable p0=num*dcrate*10+1	//start point
	Variable p1=p0-6*dcRate*10+1	//go back in time by 6s
	Variable p2=p1-10*dcRate*10+1	//go back in time by 10s relative to p1
	//linefit for baseline between p2 and p1.
	CurveFit/Q line, w[p2,p1]
	Wave W_coef
	make/FREE/D/N=(numpnts(w)) wt=w
	setscale/I x,(Leftx(w)),(rightx(w)), wt
	wt[]=wt[p]-(W_coef[0]+W_coef[1]*(p/dcrate/10))
	ps=p0
	Do
		ps+=1
	While(wt[ps]>=-noise && ps<(numpnts(wt)-2))

	return ps
	
End

//Calculates Noise based on the fist 40 seconds of the trace
Function CalcNoise(w)
	Wave w	//Data Wave
	
	Variable/G DCRate
	
	CurveFit/Q poly 3, w[0,40*DCRate*10]
	Wave W_coef
	Make/D/FREE/N=(40*DCRate) fit=(W_coef[0]+W_coef[1]*p/DCRate+W_coef[2]*(p/DCRate)^2)
	Make/D/FREE/N=(40*DCRate) copy=w
	
	copy-=fit
	
	WaveStats/Q copy
	
	Variable noise=V_max-V_min
	
	return noise
	
End

Function CleanUp(wT,wN)
Wave wT
Wave/T wN

Variable i
For(i=0;i<numpnts(wT);i+=1)
	If(stringmatch(wN(i), "Compress*"))
	DeletePoints i,1,wT,wN
	CleanUp(wT,wN)
	EndIf
EndFor
End