#pragma rtGlobals=3		// Use modern global access method and strict wave access.

//Preparation of loaded Spr raw data

Function PrepareData(num)
Variable num

	If(num)
		CalcDiffWaves(num)
		Variable/G gRef=num
	else
	
	DoWindow/K StartPanel
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=0/N=StartPanel/W=(610,277,849,473)
	SetDrawLayer UserBack
	SetDrawEnv fsize= 20,fstyle= 1
	DrawText 19,38,"Select Reference Cell"
	PopupMenu pm,pos={92,51},size={48,21}
	PopupMenu pm,mode=1,popvalue="FC1",value= #"\"FC1;FC2;FC3;FC4\""
	Button bt1,pos={52,86},size={135,67},title="Continue",font="Arial Greek"
	Button bt1,fSize=15,fColor=(0,52224,0),proc=btn
	endif
			
End

Function CalcDiffWaves(num)
	Variable num	//Number of reference flow cell (start counting from 1)
	
	Wave/T wDataRef
	//make backup of data waves
	DFREF saveDF=GetDataFolderDFR()
	
	Wave wT=$(wDataRef[0])	//Time wave
	Wave wFC1=$(wDataRef[1])
	Wave wFC2=$(wDataRef[2])
	Wave wFC3=$(wDataRef[3])
	Wave wFC4=$(wDataRef[4])
	
	If(!DataFolderExists("Backup"))
		NewDataFolder/O/S Backup
	
			Duplicate wT $(nameofwave(wT))
			Duplicate wFC1 $(nameofwave(wFC1))
			Duplicate wFC2 $(nameofwave(wFC2))
			Duplicate wFC3 $(nameofwave(wFC3))
			Duplicate wFC4 $(nameofwave(wFC4))
		
		SetDataFolder saveDF
	EndIf	
	
	Wave wRef=$(wDataRef[num])
	
	Variable sFC1=0
	Variable sFC2=0//0.3	//relative delay of flow cells relative to flow cell 1 in seconds
	Variable sFC3=0//1.5	//Only valid for 10ul/min flow!!
	Variable sFC4=0//1.85
		
	Variable/G DCRate	//data capture rate in Hz.
	
	//corrects for the delay between the flow cells
	Variable i
	If(num==1)		//FC1 is Reference
		Wave w1=$(wDataRef[num+1])
		Wave w2=$(wDataRef[num+2])
		Wave w3=$(wDataRef[num+3])
		For(i=0;i<(numpnts(wRef)-50);i+=1)
			w1[i]=w1[i+(sFC2*dcRate)]
			w2[i]=w2[i+(sFC3*dcRate)]
			w3[i]=w3[i+(sFC4*dcRate)]
		EndFor
	elseif(num==2)	//FC2 is Reference
		Wave w1=$(wDataRef[num-1])
		Wave w2=$(wDataRef[num+1])
		Wave w3=$(wDataRef[num+2])
		For(i=0;i<(numpnts(wRef)-50);i+=1)
			w1[i]=w1[i+(sFC1*dcRate)]
			w2[i]=w2[i+(sFC3*dcRate)]
			w3[i]=w3[i+(sFC4*dcRate)]
		EndFor
	elseif(num==3)	//FC3 is Reference
		Wave w1=$(wDataRef[num-2])
		Wave w2=$(wDataRef[num-1])
		Wave w3=$(wDataRef[num+1])
		For(i=0;i<(numpnts(wRef)-50);i+=1)
			w1[i]=w1[i+(sFC1*dcRate)]
			w2[i]=w2[i+(sFC2*dcRate)]
			w3[i]=w3[i+(sFC4*dcRate)]
		EndFor
	elseif(num==4)	//FC4 is Reference
		Wave w1=$(wDataRef[num-3])
		Wave w2=$(wDataRef[num-2])
		Wave w3=$(wDataRef[num-1])
		For(i=0;i<(numpnts(wRef)-50);i+=1)
			w1[i]=w1[i+(sFC1*dcRate)]
			w2[i]=w2[i+(sFC2*dcRate)]
			w3[i]=w3[i+(sFC3*dcRate)]
		EndFor
	endif
	
	make/O/D/N=(numpnts(wRef)) $(nameofwave(w1)+"Diff"), $(nameofwave(w2)+"Diff"), $(nameofwave(w3)+"Diff")
	
	Wave w1D=$(nameofwave(w1)+"Diff")
	Wave w2D=$(nameofwave(w2)+"Diff")
	Wave w3D=$(nameofwave(w3)+"Diff")
	
	//calculate differential signal by substracting
	//reference cell from data cell
	w1D=w1-wRef
	w2D=w2-wRef
	w3D=w3-wRef
	//Substract baseline
	Wavestats/Q/R=[0,40*dcrate] w1D
	w1D-=V_avg
	Wavestats/Q/R=[0,40*dcrate] w2D
	w2D-=V_avg
	Wavestats/Q/R=[0,40*dcrate] w3D
	w3D-=V_avg
	
	wDataRef[7]=nameofwave(w1D)
	wDataRef[8]=nameofwave(w2D)
	wDataRef[9]=nameofwave(w3D)
	
	//set x scale to seconds for all data/reference waves
	setscale/P x,0.1,(1/dcRate),"s", wRef,w1, w2, w3, w1D, w2D, w3D
	
	String interpol1=nameofwave(w1D)+"SS"
	String interpol2=nameofwave(w2D)+"SS"
	String interpol3=nameofwave(w3D)+"SS"
	
	//Interpolate to 100Hz 
	Interpolate2/T=2/N=(numpnts(w1D)*10)/F=1/Y=$interpol1 w1D
	Interpolate2/T=2/N=(numpnts(w2D)*10)/F=1/Y=$interpol2 w2D
	Interpolate2/T=2/N=(numpnts(w3D)*10)/F=1/Y=$interpol3 w3D
		
End

////////////////////////////////
////////////////////////////////
////////////////////////////////
////////////////////////////////


Function btn (ctrlName) : ButtonControl
	String ctrlName
	
	ControlInfo/W=StartPanel pm
	DoWindow/K StartPanel
	
	Variable/G gRef=V_Value
	
	CalcDiffWaves(V_Value)
End