#pragma rtGlobals=3		// Use modern global access method and strict wave access.


//Main
Function FitLarisaOn(w)
	Wave w
	wave xwave
	
//	Display/K=1 w

//	Make/O/D kk={3.07813,0.035,0.448,0.03,0.0002}
	Make/O/D kk={3909.3,0.16795,0.99345,0.65888,0.003808,0.002278,w[0]}	//

	InitializeLarisaOn(w,kk)
	
	FuncFit/L=(numpnts(w))/N /H="0000001" /M=2/NTHR=1 LarisaFitterOn kk  w /D/R
	
	wave cw
	duplicate/o cw cwon
End

Function InitializeLarisaOn(w,kk)
	Wave w	//Data Wave
	Wave kk	//Initial Kinetic Parameter Wave
	
	Variable Heighti=10	//Initial Nup layer height in [nm]
	Variable Heighte=10	//End Layer Height
	Variable NupImmob=8000	//Immobilisation signal of Nup in [RU]
	Variable MWL=22000	//Molweight of Ligand (Nup62=22000Da) [g/mol]
	Variable MWA=100000	//Molweight of Analyte (Imp=100000Da) [g/mol]
	
	Variable/G A=125e-9	//Concentration of Analyte for Injection
	
	Wave xwave
	
	
	If(numpnts(kk)!=7)
		Print "InitializeLarisa(w,pw): pw wrong dimension!"
		return -1
	Endif
	//Make Initial Concentration Wave
	Make/O/D/N=(numpnts(w),4) cw=NaN
	//Set x scaling for Concentration wave to match input data wave w
//	SetScale/I x,leftx(w),rightx(w),cw
	//Dimension Labels for easy access
	SetDimLabel 1,0,L,cw
	SetDimLabel 1,1,LAnoc,cw
	SetDimLabel 1,2,LAnc,cw
	SetDimLabel 1,3,LpAnc,cw
	
	//Set Initial Concentrations
	cw[0][%L] = NupImmob/(1.3*MWL*Heighti)
	cw[0][%LAnoc] = 0
	cw[0][%LAnc] = 0
	cw[0][%LpAnc] = 0
	
End

//Fit Function
Function LarisaFitterOn(pw,yw,xw) : FitFunc
	Wave pw,yw,xw
	
	Wave cw, kk

	Variable Heighte=10	//End Layer Height
	Variable MWA=100000	//Molweight of Analyte (Imp=100000Da) [g/mol]
	
	Wave xwave
	
	IntegrateODE/M=0 ModelLarisaOn, pw, cw

	yw[]=pw[6]+((cw[p][%LAnoc]+cw[p][%LAnc]+cw[p][%LpAnc])*MwA*Heighte*1.3) //pw[6]=offset
End


//ODE Solver
Function ModelLarisaOn(pw, tt, yw, dydt)
	Wave pw 		// Parameter wave: pw[0]=ka, pw[1]=kd, pw[2]=k1,
					// pw[3]=km1, pw[4]=k2, pw[5]=km2
	Variable tt	// Time value at which to calculate derivatives
	Wave yw		// Concentration wave: yw[0]=L, yw[1]=LAnoc,
					// yw[2]=LAnc, yw[3]=LpAnc
					// Concentration is given in Molar.
	Wave dydt		// Wave to receive dL/dt, dLAnoc/dt, etc. (output)
	

	Variable/G A	//Concentration of Analyte (constant)
	
	
	dydt[0] = pw[1]*yw[1]-pw[0]*yw[0]*A	//L
	dydt[1] = pw[0]*yw[0]*A+pw[3]*yw[2]-(pw[1]+pw[2])*yw[1]	//LAnoc
	dydt[2] = pw[2]*yw[1]+pw[5]*yw[3]-(pw[3]+pw[4])*yw[2]	//LAnc
	dydt[3] = pw[4]*yw[2]-pw[5]*yw[3]		//LpAnc
	
End
