#pragma rtGlobals=3		// Use modern global access method and strict wave access.

//This is the main procedure of the spr-auswertung procs. You only have to load this. All important
//parameters have to be set in here!

//Only working for 10ul/min flow


#include ":FileLoader"	//Contains Functions used for spr data file loading.
#include ":HelperFunctions"	//Contains small functions to help in analysis and data handling.
#include ":GetBindingCurves"
#include ":DataPrep"
#include ":SimpleOnFitting"
#include ":SimpleOffFitting"
#include ":SchochProc"
#include ":BaseLineCorrection"
#include ":MergeExperiments"
#include ":Extensions"

Variable V_fitOptions = 4	//suppress progress window
Variable V_FitError = 0 //prevent abort on error

Menu "SPR Analysis"
	"Load and Prepare",/Q,DoAll(0)
	"Load and Prepare calc Conc",/Q,DoAll(1)
	"Load Experiment/1",/Q,LoadDataFile()
	"Load Sample Name and Concentration",/Q,LoadSamplesFile(0)
	"Load Sample Name and Concentration calc",/Q,LoadSamplesFile(1)
	"Choose Working Folder",/Q,ChooseActiveDF(1)
	"Prepare Data",/Q,PrepareData(0)
	Submenu "Other Functions"
		"Restore Original Data",/Q,restore()
		"GetTimes",/Q,GetTimes()
		"Review Times",/Q,ReviewTimes()
		"Simple Equilibrium Fit",/Q,SimpleOnFit()		
		"Simple Off Fit",/Q,SimpleOffFit()
		"Merge two Experiments/2",/Q,MergeExperiments()
		"Check Baseline Drift",/Q,GetDrift()
		"Check for BSA binding",/Q,ExtractBSABinding()
		"Manual Off Fit/3",/Q,ManualOffFit()
		"Calc Analyte Surface density",/Q,CalcBoundAnalyteSurfaceDensity()
	End
	Submenu "Schoch"
		"Calculate Heights from BSA injections",/Q,GetBSATimes()
	End
End
	
Function DoAll(num)	
	Variable num	//if 0: read conc from file; if 1: read conc from file but only take first (highest conc) and calc dilutions.

	String DataFile1=DoOpenMultiFileDialog()
	String EventsFile1=DoOpenMultiFileDialogEvents()
	String SampleFile1=DoOpenMultiFileDialogEvents()
	String ExpName1
	Prompt ExpName1, "Experiment Folder Name (e.g Experiment Date)"
	DoPrompt "Give Foldername...",ExpName1
	String DataFile2=DoOpenMultiFileDialog()
	String EventsFile2=DoOpenMultiFileDialogEvents()
	String SampleFile2=DoOpenMultiFileDialogEvents()
	String ExpName2
	Prompt ExpName2, "Experiment Folder Name (e.g Experiment Date)"
	DoPrompt "Give Foldername...",ExpName2
	
	
	

	
	String path1=LoadDataFile(DataF=DataFile1,EventF=EventsFile1,expN=ExpName1)
	String path2=LoadDataFile(DataF=DataFile2,EventF=EventsFile2,expN=ExpName2)
	LoadSamplesFile(num,pathtoFolder=path1,pathtoFile=SampleFile1)
	LoadSamplesFile(num,pathtoFolder=path2,pathtoFile=SampleFile2)
	MergeExperiments(path1=path1,path2=path2,Fname="Merged")
	PrepareData(1)
	GetTimes()
	Wave w41=GetBSATimes(numData=4,numRef=1)
	Wave w31=GetBSATimes(numData=3,numRef=1)
	Wave w42=GetBSATimes(numData=4,numRef=2)
	Wave w32=GetBSATimes(numData=3,numRef=2)
	
	print "Avg init height FC3 = ", (w31[0]+w32[0])/2
	print "Avg init height FC4 = ", (w41[0]+w42[0])/2
	
	///calculate concentrations if num=1
	If(num)
	Wave Concs
	Variable i
	
	For(i=0;i<numpnts(Concs)-1;i+=1)
		Concs[i]=Concs[numpnts(Concs)-1]*2^(-(numpnts(Concs)-1-i))
	EndFor
	EndIf
End