#pragma rtGlobals=3		// Use modern global access method and strict wave access.

//Tests For Fitting


//Make Input and Output Waves

Function Larisa1(ka,kd,k1,km1,k2,km2,wc,inject,offset,num)
	Variable ka,kd,k1,km1,k2,km2,inject
	Wave wc	//Contains Injected concentrations of Analyte
	Variable offset
	Variable num	//display result

	Variable Heighti=10	//Initial Nup layer height in [nm]
	Variable Heighte=10	//End Layer Height
	Variable NupImmob=8000	//Immobilisation signal of Nup in [RU]
	Variable MWL=22000	//Molweight of Ligand (Nup62=22000Da) [g/mol]
	Variable MWA=100000	//Molweight of Analyte (Imp=100000Da) [g/mol]
	
	Variable/G A	//Concentration of Analyte for Injection

	//Ligand (FG-domain) density with a initial layer height of 10nm:
	//LDensity=NupImmobsignal/(1.3*MolweightLigand[g/mol]*height[nm])
	
	Variable LCinitial=NupImmob/(1.3*MWL*Heighti)	//Initial Ligand concentration
	
	Make/D/O ParamIn={ka,kd,k1,km1,k2,km2}
	
	Variable i
	String wOutOn,wOutOnRU,wOutOff,wOutOffRU,ROutOn,ROutOff,wOutOffOld

	if(num)
	DoWindow/K Out
	Display/W=(50,50,1231.5,527.75)/N=Out/K=1
	endif
	For(i=0;i<numpnts(wc);i+=1)
	

		
		wOutOn="KinOutOn"+num2str(i)
		wOutOffOld="KinOutOff"+num2str(i)
		If(i)
			wOutOffOld="KinOutOff"+num2str(i-1)
		EndIf
		wOutOff="KinOutOff"+num2str(i)
		wOutOnRU="KinOutOnRU"+num2str(i)
		wOutOffRU="KinOutOffRU"+num2str(i)
		ROutOn="ROutOn"+num2str(i)
		ROutOff="ROutOff"+num2str(i)
		Make/D/O/N=(inject,4) $wOutOn,$wOutOnRU,$wOutOff,$wOutOffRU
		Wave w=$wOutOn
		Wave wOld=$wOutOffOld
		Wave wRU=$wOutOnRU
		Wave wO=$wOutOff
		Wave wORU=$wOutOffRU
		Make/D/O/N=(inject) $ROutOn,$ROutOff
		Wave outOn=$ROutOn
		Wave outOff=$ROutOff
		SetDimLabel 1,0,L,w,wO
		SetDimLabel 1,1,LAnoc,w,wO
		SetDimLabel 1,2,LAnc,w,wO
		SetDimLabel 1,3,LpAnc,w,wO

		
		A=wc[i]
		If(i==0)
			w[0][%L] = LCinitial
			w[0][%LAnoc] = 0
			w[0][%LAnc] = 0
			w[0][%LpAnc] = 0


		else
			w[0][%L] = wOld[inject-1][%L]	//Initial Ligand concentration
			w[0][%LAnoc] = wOld[inject-1][%LAnoc]
			w[0][%LAnc] = wOld[inject-1][%LAnc]
			w[0][%LpAnc] = wOld[inject-1][%LpAnc]

		endif
		
		
		setscale/I x,0,inject,w

		IntegrateODE/M=0 ModelLarisa1, ParamIn, w
		
		print "V_ODETotalSteps",V_ODETotalSteps
		print "V_ODEStepSize",V_ODEStepSize
		print "V_ODEMinStep",V_ODEMinStep
		print "V_ODEFunctionCalls",V_ODEFunctionCalls
		
		
		Duplicate/O w wRU
		wRU=w*MwA*Heighte*1.3
		 
		setscale/I x,(i*2*inject),(i*2*inject+inject),wRU
		
		A=0
		
		wO[0][%L] = w[inject-1][%L]	//Initial Ligand concentration
		wO[0][%LAnoc] = w[inject-1][%LAnoc]
		wO[0][%LAnc] = w[inject-1][%LAnc]
		wO[0][%LpAnc] = w[inject-1][%LpAnc]

		
		
		IntegrateODE/M=0 ModelLarisa1, ParamIn, wO
		
		
		
		Duplicate/O wO wORU
		wORU=wO*MwA*Heighte*1.3
		setscale/I x,(i*2*inject+inject),(i*2*inject+2*inject),wORU
		
		outOn[]=offset+wRU[p][%LAnoc]+wRU[p][%LAnc]+wRU[p][%LpAnc]
		setscale/I x,(i*2*inject),(i*2*inject+inject),outOn
		outOff[]=offset+wORU[p][%LAnoc]+wORU[p][%LAnc]+wORU[p][%LpAnc]
		setscale/I x,(i*2*inject+inject),(i*2*inject+2*inject),outOff
		
		if(num)
//		AppendToGraph wRU[][%L]
		AppendToGraph wRU[][%LAnoc]
		AppendToGraph wRU[][%LAnc]
		AppendToGraph wRU[][%LpAnc]
//		ModifyGraph lsize($(nameofwave(wRU)+"#4"))=1.5,rgb($(nameofwave(wRU)+"#4"))=(0,0,65280)
		AppendToGraph outOn
		ModifyGraph lsize($nameofwave(outOn))=2.5,rgb($nameofwave(outOn))=(0,0,0)
//		AppendToGraph wORU[][%L]
		AppendToGraph wORU[][%LAnoc]
		AppendToGraph wORU[][%LAnc]
		AppendToGraph wORU[][%LpAnc]
//		ModifyGraph lsize($(nameofwave(wORU)+"#4"))=1.5,rgb($(nameofwave(wORU)+"#4"))=(0,0,65280)
		AppendToGraph outOff
		ModifyGraph lsize($nameofwave(outOff))=2.5,rgb($nameofwave(outOff))=(0,0,0)
		
		endif
	EndFor

End


Function ModelLarisa1(pw, tt, yw, dydt)
	Wave pw 		// Parameter wave: pw[0]=ka, pw[1]=kd, pw[2]=k1,
					// pw[3]=km1, pw[4]=k2, pw[5]=km2
	Variable tt	// Time value at which to calculate derivatives
	Wave yw		// Concentration wave: yw[0]=L, yw[1]=LAnoc,
					// yw[2]=LAnc, yw[3]=LpAnc
					// Concentration is given in Molar.
	Wave dydt		// Wave to receive dL/dt, dLAnoc/dt, etc. (output)
	

	Variable/G A	//Concentration of Analyte (constant)
	
	Variable p0y0A=pw[0]*yw[0]*A
	Variable p1y1=pw[1]*yw[1]
	Variable p3y2=pw[3]*yw[2]
	Variable p2y1=pw[2]*yw[1]
	Variable p5y3=pw[5]*yw[3]
	Variable p4y2=pw[4]*yw[2]
	
	
	dydt[0] = p1y1-p0y0A	//L
	dydt[1] = p0y0A+p3y2-p1y1-p2y1	//LAnoc
	dydt[2] = p2y1+p5y3-p3y2-p4y2	//LAnc
	dydt[3] = p4y2-p5y3	//LpAnc
	
End

