#pragma rtGlobals=3		// Use modern global access method and strict wave access.

//Derivative function of coupled first-order equations
//of Impbeta binding to FG-Nups.
//3-Step-Model from Larisa
//Conversion factor used: 1300RU = 1ng/mm2


//Make Input and Output Waves

Function Start(ka1,kd1,ka2,kd2,k1,km1,k2,km2,k3,km3,wc,inject)
	Variable ka1,kd1,ka2,kd2,k1,km1,k2,km2,k3,km3,inject
	Wave wc	//Contains Injected concentrations of Analyte
	
	Variable Heighti=10	//Initial Nup layer height in [nm]
	Variable Heighte=10	//End Layer Height
	Variable NupImmob=8000	//Immobilisation signal of Nup in [RU]
	Variable MWL=22000	//Molweight of Ligand (Nup62=22000Da) [g/mol]
	Variable MWA=100000	//Molweight of Analyte (Imp=100000Da) [g/mol]
	Variable P0max=10000	//Maximal binding capacity for layer 0 in RU (1000RU=1Layer of Imp)
	
	Variable/G A	//Concentration of Analyte for Injection

	//Ligand (FG-domain) density with a initial layer height of 10nm:
	//LDensity=NupImmobsignal/(1.3*MolweightLigand[g/mol]*height[nm])
	
	Variable LCinitial=NupImmob/(1.3*MWL*Heighti)	//Initial Ligand concentration
	Variable P0Cinitial=P0max/(1.3*MWA*10)	//Initial "binding sites" in layer 0.
	
	Make/D/O ParamIn={ka1,kd1,ka2,kd2,k1,km1,k2,km2,k3,km3}
	
	Variable i
	String wOutOn,wOutOnRU,wOutOff,wOutOffRU,ROutOn,ROutOff,wOutOffOld
	
	NewPanel /N=ProgressPanel /W=(285,111,739,193)
	ValDisplay valdisp0,pos={18,32},size={342,18},limits={0,(numpnts(wc)),0},barmisc={0,0}
	ValDisplay valdisp0,value= _NUM:0
	ValDisplay valdisp0,mode= 3
	ValDisplay valdisp0,highColor=(0,65535,0)
	Button bStop,pos={375,32},size={50,20},title="Stop"
	DoUpdate /W=ProgressPanel /E=1	// mark this as our progress window
	
	DoWindow/K Out
	Display/W=(50,50,1231.5,527.75)/N=Out/K=1
	
	For(i=0;i<numpnts(wc);i+=1)
	

		
		wOutOn="KinOutOn"+num2str(i)
		wOutOffOld="KinOutOff"+num2str(i)
		If(i)
			wOutOffOld="KinOutOff"+num2str(i-1)
		EndIf
		wOutOff="KinOutOff"+num2str(i)
		wOutOnRU="KinOutOnRU"+num2str(i)
		wOutOffRU="KinOutOffRU"+num2str(i)
		ROutOn="ROutOn"+num2str(i)
		ROutOff="ROutOff"+num2str(i)
		Make/D/O/N=(inject,6) $wOutOn,$wOutOnRU,$wOutOff,$wOutOffRU
		Wave w=$wOutOn
		Wave wOld=$wOutOffOld
		Wave wRU=$wOutOnRU
		Wave wO=$wOutOff
		Wave wORU=$wOutOffRU
		Make/D/O/N=(inject) $ROutOn,$ROutOff
		Wave outOn=$ROutOn
		Wave outOff=$ROutOff
		SetDimLabel 1,0,L,w,wO
		SetDimLabel 1,1,LAn,w,wO
		SetDimLabel 1,2,LAc,w,wO
		SetDimLabel 1,3,LAnc,w,wO
		SetDimLabel 1,4,P,w,wO
		SetDimLabel 1,5,PLAnc,w,wO
		
		A=wc[i]
		If(i==0)
			w[0][%L] = LCinitial
			w[0][%LAn] = 0
			w[0][%LAc] = 0
			w[0][%LAnc] = 0
			w[0][%P] = P0Cinitial
			w[0][%PLAnc] = 0
		else
			w[0][%L] = wOld[inject-1][%L]	//Initial Ligand concentration
			w[0][%LAn] = wOld[inject-1][%LAn]
			w[0][%LAc] = wOld[inject-1][%LAc]
			w[0][%LAnc] = wOld[inject-1][%LAnc]
			w[0][%P] = wOld[inject-1][%P]
			w[0][%PLAnc] = wOld[inject-1][%PLAnc]
		endif
		
		
		setscale/I x,0,inject,w

		IntegrateODE/M=1 Model3, ParamIn, w
		
		
		
		
		Duplicate/O w wRU
		wRU=w*MwA*Heighte*1.3
		 
		setscale/I x,(i*2*inject),(i*2*inject+inject),wRU
		
		A=0
		
		wO[0][%L] = w[inject-1][%L]	//Initial Ligand concentration
		wO[0][%LAn] = w[inject-1][%LAn]
		wO[0][%LAc] = w[inject-1][%LAc]
		wO[0][%LAnc] = w[inject-1][%LAnc]
		wO[0][%P] = w[inject-1][%P]
		wO[0][%PLAnc] = w[inject-1][%PLAnc]
		
		
		IntegrateODE/M=1 Model3, ParamIn, wO
		
		
		
		Duplicate/O wO wORU
		wORU=wO*MwA*Heighte*1.3
		setscale/I x,(i*2*inject+inject),(i*2*inject+2*inject),wORU
		
		outOn[]=wRU[p][%LAn]+wRU[p][%LAc]+wRU[p][%LAnc]+wRU[p][%PLAnc]
		setscale/I x,(i*2*inject),(i*2*inject+inject),outOn
		outOff[]=wORU[p][%LAn]+wORU[p][%LAc]+wORU[p][%LAnc]+wORU[p][%PLAnc]
		setscale/I x,(i*2*inject+inject),(i*2*inject+2*inject),outOff
		
		AppendToGraph wRU[][%L]
		AppendToGraph wRU[][%LAn]
		AppendToGraph wRU[][%LAc]
		AppendToGraph wRU[][%LAnc]
		AppendToGraph wRU[][%P]
		AppendToGraph wRU[][%PLAnc]
		ModifyGraph lsize($(nameofwave(wRU)+"#4"))=1.5,rgb($(nameofwave(wRU)+"#4"))=(0,0,65280)
		AppendToGraph outOn
		ModifyGraph lsize($nameofwave(outOn))=2.5,rgb($nameofwave(outOn))=(0,0,0)
		AppendToGraph wORU[][%L]
		AppendToGraph wORU[][%LAn]
		AppendToGraph wORU[][%LAc]
		AppendToGraph wORU[][%LAnc]
		AppendToGraph wORU[][%P]
		AppendToGraph wORU[][%PLAnc]
		ModifyGraph lsize($(nameofwave(wORU)+"#4"))=1.5,rgb($(nameofwave(wORU)+"#4"))=(0,0,65280)
		AppendToGraph outOff
		ModifyGraph lsize($nameofwave(outOff))=2.5,rgb($nameofwave(outOff))=(0,0,0)
		
		ValDisplay valdisp0,value= _NUM:i+1,win=ProgressPanel
		DoUpdate /W=ProgressPanel
	EndFor

	KillWindow ProgressPanel
End


Function Model3(pw, tt, yw, dydt)
	Wave pw 		// Parameter wave: pw[0]=ka1, pw[1]=kd1, pw[2]=ka2,
					// pw[3]=kd2, pw[4]=k1, pw[5]=km1, pw[6]=k2, pw[7]=km2, pw[8]=k3, pw[9]=km3
	Variable tt	// Time value at which to calculate derivatives
	Wave yw		// Concentration wave: yw[0]=L, yw[1]=LAn,
					// yw[2]=LAc, yw[3]=LAnc, yw[4]=P, yw[5]=PLAnc
					// Concentration is given in Molar.
	Wave dydt		// Wave to receive dAs/dt, dAsL/dt, etc. (output)
	

	Variable/G A	//Concentration of Analyte (constant)
	
	
	dydt[0] = pw[1]*yw[1]+pw[3]*yw[3]-(pw[0]+pw[2])*yw[0]*A	//L
	dydt[1] = pw[0]*yw[0]*A+pw[5]*yw[3]-(pw[1]+pw[4])*yw[1]	//LAn
	dydt[2] = pw[2]*yw[0]*A+pw[7]*yw[3]-(pw[3]+pw[6])*yw[2]	//LAc
	dydt[3] = pw[4]*yw[1]+pw[6]*yw[2]+pw[9]*yw[5]-(pw[5]+pw[7])*yw[3]-pw[8]*yw[3]*yw[4]	//LAnc
	dydt[4] = pw[9]*yw[5]-pw[8]*yw[3]*yw[4]	//P
	dydt[5] = pw[8]*yw[3]*yw[4]-pw[9]*yw[5]	//PLAnc
	
	
	
End

