#pragma rtGlobals=3		// Use modern global access method and strict wave access.


//This Function locates all (if there're any) BSA peak positions.
Function/WAVE GetBSATimes([numData,numRef,disp])
	Variable numData,numRef,disp
	
	Variable/G numBSA=3
	
	If(!ParamIsDefault(numData) && !ParamIsDefault(numRef))
	
		return Schochalization(numData,numRef,0,0)
	
	else
	
		Wave/T wDataRef	//Contains names of data waves
	
		String quote="\""
		String list
		list=quote+wDataRef[1]+";"+wDataRef[2]+";"+wDataRef[3]+";"+wDataRef[4]+";"+quote
	
		
		DoWindow/K BSAStart
		PauseUpdate; Silent 1		// building window...
		NewPanel /K=0/N=BSAStart/W=(610,277,1000,521)
		SetDrawLayer UserBack
		SetDrawEnv fsize= 20,fstyle= 1
		DrawText 19,38,"Select Flow Cells for Schochalization"
		DrawText 162,69,"Data Wave"
		DrawText 153,110,"Reference Wave"
		PopupMenu pm,pos={163,71},size={70,21}
		PopupMenu pm,mode=1,popvalue="SPRFC1",value= #"\"SPRFC1;SPRFC2;SPRFC3;SPRFC4;\""
		PopupMenu pm2,pos={163,113},size={70,21}
		PopupMenu pm2,mode=1,popvalue="SPRFC1",value= #"\"SPRFC1;SPRFC2;SPRFC3;SPRFC4;\""
		Button bt1,pos={134,149},size={135,67},proc=btnBSA,title="Continue"
		Button bt1,font="Arial Greek",fSize=15,fColor=(0,52224,0)
		SetVariable v1,value=numBSA,noproc,title=" "
		SetVariable v1 pos={30,103},size={60,17}
		SetVariable v1 noedit=1,limits={1,3,1}
		DrawText 30,90,"Number of BSA\rinjections for avg"
		DrawText 30,110,"1: first two, 2: second two, 3: all"
		CheckBox cb1,pos={30,137},size={81,14},title="BSAdrop corr",value= 0
	endif
	
	
End

Function btnBSA (ctrlName) : ButtonControl
	String ctrlName
	
	ControlInfo/W=BSAStart pm
	Variable datawave=V_Value
	ControlInfo/W=BSAStart pm2
	Variable refwave=V_Value
	
	
	ControlInfo /W=BSAStart cb1
	
	DoWindow/K BSAStart
	
	Schochalization(datawave,refwave,V_value,1)
	
End

Function/WAVE Schochalization(dnum,rnum,opt,disp)
	Variable dnum	//Data Wave from wDataRef
	Variable rnum	//Reference Wave from wDataRef
	Variable opt	//Option for BSAdrop check
	Variable disp //display results
	
	Variable/G DCRate	//Data capture rate in Hz
	Variable/G numBSA
	
	Variable numbsastart,numbsastop
	
	If(numBSA==1)
		numbsastart=0
		numbsastop=2
		numbsa=2
	elseif(numBSA==2)
		numbsastart=1
		numbsastop=3
		numbsa=2
	elseif(numBSA==3)
		numbsastart=0
		numbsastop=3
		numbsa=3
	endif
	
	If(opt)
		if(!exists("Times"))
			print "Missing wTimes: Use GetTimes Function first!"
		else
			Wave wT=Times
		endif
	Endif
	
	Variable/G gToffset
	Variable/G gPoffset
	
	if(!exists("gToffset"))
		gToffset=0
		gPoffset=0
	endif
	

	Wave/T wDataRef	//Contains names of data waves
	Wave/T wName = $(wDataRef[5])	//Contains Name of event (from logfile)
	Wave wTime = $(wDataRef[6])	//Contains Time of event (from logfile)
	Wave w = $(wDataRef[dnum])		//Data Wave
	Wave wr = $(wDataRef[rnum])	//Reference Wave
	Wave wdiff=$(wDataRef[dnum]+"Diff")
	
	If(!waveexists(w))
		print "Ups: Schochalization(num) did not find any data wave!"
		return w
	Endif
	
	DFREF saveDF=GetDataFolderDFR()
	
	String cmd = "NewDataFolder/O/S BSAFC"+num2str(dnum)+"vsFC"+num2str(rnum)
	
	Execute/Q/Z cmd
		
	make/D/O/N=(0,3) BSATimes		//It is assumed that a single height measurement
									//is compsed of three BSA injections wrapped in a
									//single Injection Sequence (single cycle kinetics)
	
	//Extract BSA injection Start times from Logfile
	Variable i,j,k,offset=0,count=0
	Variable delta	//seconds from injection ready to middle of two injections.
	For(i=0;i<(numpnts(wName));i+=1)
	
		If(stringmatch(wname(i),"Flow path:*"))
			offset=count*gToffset
			count+=1
		EndIf
		
		If(stringmatch(wName(i), "Inject sequence: Start*"))
			Redimension /N=(dimsize(BSATimes,0)+1,-1) BSATimes
			k=0
			For(j=i;j<(numpnts(wName));j+=1)	
				If(stringmatch(wName(j),"Inject: Start*"))
					If(k==1)
						delta=wTime[j+1]-wTime[j]
					EndIf
					If(k<3)//Only first 3 BSA injections are taken into calculation
						BSATimes[dimsize(BSATimes,0)-1][k]=wTime[j]//+offset
					//	print "gToffset=",gToffset,"Line=",i,"PeakNumber=",j,"Time=",wTime[j],"offset=",offset,"Out=",wTime[j]+offset
					elseif(k>=3)
						print "More than 3 BSA injections found! ("+num2str(wTime[j])+"seconds"
					EndIf
					k+=1
				Elseif(stringmatch(wName(j),"Inject sequence: Ready*"))
					break
				EndIf
			EndFor
		Endif		
	EndFor
	
	
	
	//Find BSA Peak
	make/D/O/N=(numbsa) AmpRefTemp=Nan, AmpDataTemp=Nan
	make/D/O/N=0 AvgBSAPeakAmpData
	make/D/O/N=0 AvgBSAPeakAmpDataError
	make/D/O/N=0 AvgBSAPeakAmpRef
	make/D/O/N=0 AvgBSAPeakAmpRefError
	make/D/O/N=(0,24) BSAPeakPos //positions of BSA peak used for DisplayBSAPeakPos
									//[first point of peak, second point of peak, first point of baseline, second point of baseline]Peak 1, Peak2 ...
	If(opt)
		make/D/O/N=0 wBSADrop
	EndIf
	For(i=0;i<dimsize(BSATimes,0);i+=1)
		Redimension /N=(numpnts(AvgBSAPeakAmpData)+1) AvgBSAPeakAmpData
		Redimension /N=(numpnts(AvgBSAPeakAmpDataError)+1) AvgBSAPeakAmpDataError
		Redimension /N=(numpnts(AvgBSAPeakAmpRef)+1) AvgBSAPeakAmpRef
		Redimension /N=(numpnts(AvgBSAPeakAmpRefError)+1) AvgBSAPeakAmpRefError
		Redimension /N=(dimsize(BSAPeakPos,0)+1,-1) BSAPeakPos
		For(j=numbsastart;j<numbsastop;j+=1)	//change to j=1 to include only last two BSA injections for calc.
			Wave BSATemp=GetBSAPeak((BSATimes[i][j])*Dcrate-2*DcRate,w,delta)
			AmpDataTemp[j]=BSATemp[0]
		//	AvgBSAPeakAmpData[numpnts(AvgBSAPeakAmpData)-1]+=BSATemp[0]
		//	AvgBSAPeakAmpDataError[numpnts(AvgBSAPeakAmpData)-1]+=(BSATemp[5])^2
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][j*4]=BSATemp[1]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+1]=BSATemp[2]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+2]=BSATemp[3]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+3]=BSATemp[4]
			Wave BSATemp=GetBSAPeak((BSATimes[i][j])*DCrate-2*DcRate,wr,delta)
			AmpRefTemp[j]=BSATemp[0]
		//	AvgBSAPeakAmpRef[numpnts(AvgBSAPeakAmpRef)-1]+=BSATemp[0]
		//	AvgBSAPeakAmpRefError[numpnts(AvgBSAPeakAmpRef)-1]+=(BSATemp[5])^2
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][j*4+12]=BSATemp[1]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+13]=BSATemp[2]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+14]=BSATemp[3]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+15]=BSATemp[4]
		EndFor
		Wavestats/Q AmpDataTemp
		AvgBSAPeakAmpData[numpnts(AvgBSAPeakAmpData)-1]=V_avg	//change to /=2 when calc only last two BSA injections.
		AvgBSAPeakAmpDataError[numpnts(AvgBSAPeakAmpData)-1]=V_sdev
		Wavestats/Q AmpRefTemp
		AvgBSAPeakAmpRef[numpnts(AvgBSAPeakAmpRef)-1]=V_avg
		AvgBSAPeakAmpRefError[numpnts(AvgBSAPeakAmpRef)-1]=V_sdev
		
	//	AvgBSAPeakAmpDataError[numpnts(AvgBSAPeakAmpData)-1]=sqrt(AvgBSAPeakAmpDataError[numpnts(AvgBSAPeakAmpData)-1]/numBSA)
	//	AvgBSAPeakAmpRefError[numpnts(AvgBSAPeakAmpRef)-1]=sqrt(AvgBSAPeakAmpRefError[numpnts(AvgBSAPeakAmpRef)-1]/numBSA)
	
	
		If(opt)
			Redimension /N=(dimsize(wBSADrop,0)+1,-1) wBSADrop
			If(i==0)
				wBSADrop[numpnts(wBSADrop)-1]=wdiff[0]-wdiff[BSAPeakPos[i][2]]
				print nameofwave(wdiff),wdiff[0],wdiff[BSAPeakPos[i][2]],BSAPeakPos[i][2]
				
			else
				wBSADrop[numpnts(wBSADrop)-1]=wdiff[wT[i-1][2]*Dcrate]-wdiff[BSAPeakPos[i][2]]
			endif
		EndIf
	EndFor
	
	make/D/O/N=(numpnts(AvgBSAPeakAmpData)) AvgHeight
	make/D/O/N=(numpnts(AvgBSAPeakAmpData)) AvgHeightError
	AvgHeight=350/2*ln(AvgBSAPeakAmpRef/AvgBSAPeakAmpData)+1	//+1 when proteins are directly linked to the surface.
	
	For(i=0;i<numpnts(AvgHeightError);i+=1)
	
		AvgHeightError[i]=sqrt((175/AvgBSAPeakAmpRef[i]*AvgBSAPeakAmpRefError[i])^2+(-175/AvgBSAPeakAmpData[i]*AvgBSAPeakAmpDataError[i])^2)
	
	EndFor
	
	String wAvgHeight=getdatafolder(1)+"AvgHeight"
	String wAvgHeightError=getdatafolder(1)+"AvgHeightError"
	String folder=getdatafolder(1)
	SetDataFolder saveDF
	
	Wave Concs
	If(exists("Concs"))
		Make/D/O/N=(numpnts(Concs)+1) ConcsforHeight=0
			For(i=0;i<numpnts(Concs);i+=1)
				ConcsforHeight[i+1]=Concs[i]
			EndFor
		if(disp)
		Display/K=1 $wAvgHeight vs ConcsforHeight
		Label bottom "Protein Concentration [M]"
		endif
	else
		if(disp)
		Display/K=1 $wAvgHeight
		Label bottom "Injection Number"
		endif
	endif
	if(disp)
	ErrorBars AvgHeight Y,wave=($wAvgHeightError,$wAvgHeightError)
	
	String labl="\s("+nameofwave($wAvgHeight)+") Average Height\r"+nameofwave(w)+" vs "+nameofwave(wr)
	
	ModifyGraph mode=4,marker=41,rgb=(13056,0,26112)
	ModifyGraph mirror=1,prescaleExp(bottom)=6;DelayUpdate
	Label left "Layer Height [nm]";DelayUpdate
	TextBox/A=RT/N=text0 labl
	ModifyGraph log(bottom)=1
	endif
	
	return $wAvgHeight
End

Function/WAVE GetBSAPeak(p,w,delta)
	Variable p	//Point number where peaksearch starts.
	Wave w	//Data Wave
	Variable delta	//add delta to found peak pos to get baseline
	
//	Wave BSATemp
//	If(!waveexists(BSATemp))
		Make/O/D/N=6 BSATemp=NaN	//BSATemp[0]=Peak Height, BSATemp[1] pos 1 peak, BSATemp[2] pos 2 peak
									//BSATemp[3] and [4] pos 1 and 2 for baseline.
									//[5] error in amplitude (RU of heigtht)
//	else
//		BSATemp=NaN
//	endif
	
	NVAR rate = ::DCRate	//Data capture rate in Hz
	
	Variable Baseline=w[p]
	
	
	Variable i,j,pb=p
	
	Variable peak,error
	
	FindPeak /M=(Baseline+100) /P /Q /R=[p,(p+20000)] w
	
	Variable peak_avg=0.1 //10% of peakwidth is used for peak height avg calc
	Variable base_avg=1.3	//30% of baseline width is used to calc avg baseline
	
	
	If(!V_flag)
		p=floor(V_PeakLoc)
		i=p+1
		Do
			If(w[i]-w[p]<-5)			
				
				For(j=0;j<200;j+=1)
					If(abs(w[i-j]-w[p-j])<=1)
						wavestats /Q /R=[(p-j)-((p-j)-pb)*peak_avg,(p-j)] w //0.1
						BSATemp[1]=(p-j)-((p-j)-pb)*peak_avg	//first point of peak
						BSATemp[2]=(p-j)						//second point of peak
						peak=V_avg
						error=V_sdev
						wavestats /Q /R=[(p-j)+(delta*rate),(p-j)+(delta*rate)*base_avg] w	//1.3
						BSATemp[3]=(p-j)+(delta*rate)	//first point of baseline
						BSATemp[4]=(p-j)+(delta*rate)*base_avg	//second point of baseline
						peak-=V_avg
						//error=sqrt((error-V_sdev)^2)
						break
					EndIf
				EndFor
				
				break
			EndIf
			i+=1
			p+=1
		While(p<(numpnts(w)-10)) //p<p+20000 || 
		
	Else
		print "No BSA Peak was found! Point:",pb
		print Baseline, Baseline+100, V_flag
		print nameofwave(w),w[p]
	EndIf
	BSATemp[0]= peak
	BSATemp[5]= error
	return BSATemp
End