#pragma rtGlobals=3		// Use modern global access method and strict wave access.


//This Function locates all (if there're any) BSA peak positions.
Function GetBSATimes()

	Wave/T wDataRef	//Contains names of data waves
	
	String quote="\""
	String list
	list=quote+wDataRef[1]+";"+wDataRef[2]+";"+wDataRef[3]+";"+wDataRef[4]+";"+quote
	
	DoWindow/K BSAStart
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=0/N=BSAStart/W=(610,277,1000,521)
	SetDrawLayer UserBack
	SetDrawEnv fsize= 20,fstyle= 1
	DrawText 19,38,"Select Flow Cells for Schochalization"
	DrawText 162,69,"Data Wave"
	DrawText 153,110,"Reference Wave"
	PopupMenu pm,pos={163,71},size={70,21}
	PopupMenu pm,mode=1,popvalue="SPRFC1",value= #"\"SPRFC1;SPRFC2;SPRFC3;SPRFC4;\""
	PopupMenu pm2,pos={163,113},size={70,21}
	PopupMenu pm2,mode=1,popvalue="SPRFC1",value= #"\"SPRFC1;SPRFC2;SPRFC3;SPRFC4;\""
	Button bt1,pos={134,149},size={135,67},proc=btnBSA,title="Continue"
	Button bt1,font="Arial Greek",fSize=15,fColor=(0,52224,0)
	
End

Function btnBSA (ctrlName) : ButtonControl
	String ctrlName
	
	ControlInfo/W=BSAStart pm
	Variable datawave=V_Value
	ControlInfo/W=BSAStart pm2
	Variable refwave=V_Value
	DoWindow/K BSAStart
	
	Schochalization(datawave,refwave)
End

Function Schochalization(dnum,rnum)
	Variable dnum	//Data Wave from wDataRef
	Variable rnum	//Reference Wave from wDataRef
	
	Variable/G DCRate	//Data capture rate in Hz

	Wave/T wDataRef	//Contains names of data waves
	Wave/T wName = $(wDataRef[5])	//Contains Name of event (from logfile)
	Wave wTime = $(wDataRef[6])	//Contains Time of event (from logfile)
	Wave w = $(wDataRef[dnum])		//Data Wave
	Wave wr = $(wDataRef[rnum])	//Reference Wave
	
	If(!waveexists(w))
		print "Ups: Schochalization(num) did not find any data wave!"
		return -1
	Endif
	
	DFREF saveDF=GetDataFolderDFR()
	
	String cmd = "NewDataFolder/O/S BSAFC"+num2str(dnum)+"vsFC"+num2str(rnum)
	
	Execute/Q/Z cmd
		
	make/O/N=(0,3) BSATimes		//It is assumed that a single height measurement
									//is compsed of three BSA injections wrapped in a
									//single Injection Sequence (single cycle kinetics)
	
	//Extract BSA injection Start times from Logfile
	Variable i,j,k
	Variable delta	//seconds from injection ready to middle of two injections.
	For(i=0;i<(numpnts(wName));i+=1)
		If(stringmatch(wName(i), "Inject sequence: Start*"))
			Redimension /N=(dimsize(BSATimes,0)+1,-1) BSATimes
			k=0
			For(j=i;j<(numpnts(wName));j+=1)	
				If(stringmatch(wName(j),"Inject: Start*"))
					If(k==1)
						delta=wTime[j+1]-wTime[j]
					EndIf
					If(k<3)//Only first 3 BSA injections are taken into calculation
						BSATimes[dimsize(BSATimes,0)-1][k]=wTime[j]
					elseif(k>=3)
						print "More than 3 BSA injections found! ("+num2str(wTime[j])+"seconds"
					EndIf
					k+=1
				Elseif(stringmatch(wName(j),"Inject sequence: Ready*"))
					break
				EndIf
			EndFor
		Endif		
	EndFor
	
	
	
	//Find BSA Peak
	make/O/N=0 AvgBSAPeakAmpData
	make/O/N=0 AvgBSAPeakAmpRef
	make/O/N=(0,24) BSAPeakPos //positions of BSA peak used for DisplayBSAPeakPos
									//[first point of peak, second point of peak, first point of baseline, second point of baseline]Peak 1, Peak2 ...
	For(i=0;i<dimsize(BSATimes,0);i+=1)
		Redimension /N=(numpnts(AvgBSAPeakAmpData)+1) AvgBSAPeakAmpData
		Redimension /N=(numpnts(AvgBSAPeakAmpRef)+1) AvgBSAPeakAmpRef
		Redimension /N=(dimsize(BSAPeakPos,0)+1,-1) BSAPeakPos	
		For(j=1;j<3;j+=1)	//change to j=1 to include only last two BSA injections for calc.
			Wave BSATemp=GetBSAPeak((BSATimes[i][j])*Dcrate-2*DcRate,w,delta)
			AvgBSAPeakAmpData[numpnts(AvgBSAPeakAmpData)-1]+=BSATemp[0]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][j*4]=BSATemp[1]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+1]=BSATemp[2]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+2]=BSATemp[3]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+3]=BSATemp[4]
			Wave BSATemp=GetBSAPeak((BSATimes[i][j])*DCrate-2*DcRate,wr,delta)
			AvgBSAPeakAmpRef[numpnts(AvgBSAPeakAmpRef)-1]+=BSATemp[0]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][j*4+12]=BSATemp[1]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+13]=BSATemp[2]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+14]=BSATemp[3]
			BSAPeakPos[dimsize(BSAPeakPos,0)-1][(j*4)+15]=BSATemp[4]
		EndFor
		AvgBSAPeakAmpData[numpnts(AvgBSAPeakAmpData)-1]/=2	//change to /=2 when calc only last two BSA injections.
		AvgBSAPeakAmpRef[numpnts(AvgBSAPeakAmpRef)-1]/=2
	EndFor
	
	make/O/N=(numpnts(AvgBSAPeakAmpData)) AvgHeight
	AvgHeight=350/2*ln(AvgBSAPeakAmpRef/AvgBSAPeakAmpData)+1	//+1 when proteins are directly linked to the surface.
		
	SetDataFolder saveDF
End

Function/WAVE GetBSAPeak(p,w,delta)
	Variable p	//Point number where peaksearch starts.
	Wave w	//Data Wave
	Variable delta	//add delta to found peak pos to get baseline
	
	Wave BSATemp
	If(!waveexists(BSATemp))
		Make/N=5 BSATemp=NaN	//BSATemp[0]=Peak Height, BSATemp[1] pos 1 peak, BSATemp[2] pos 2 peak
									//BSATemp[3] and [4] pos 1 and 2 for baseline.
	else
		BSATemp=NaN
	endif
	
	NVAR rate = ::DCRate	//Data capture rate in Hz
	
	Variable Baseline=w[p]
	
	
	Variable i,j,pb=p
	
	Variable peak
	
	FindPeak /M=(Baseline+100) /P /Q /R=[p,(p+20000)] w
	
	If(!V_flag)
		p=V_PeakLoc
		i=p+1
		Do
			If(w[i]-w[p]<-2)
				For(j=0;j<200;j+=1)
					If(abs(w[i-j]-w[p-j])<=1)
						wavestats /Q /R=[(p-j)-((p-j)-pb)*0,(p-j)] w //0.1
						BSATemp[1]=(p-j)-((p-j)-pb)*0	//first point of peak
						BSATemp[2]=(p-j)						//second point of peak
						peak=V_avg
						wavestats /Q /R=[(p-j)+(delta*rate),(p-j)+(delta*rate)*0] w	//1.3
						BSATemp[3]=(p-j)+(delta*rate)	//first point of baseline
						BSATemp[4]=(p-j)+(delta*rate)*0	//second point of baseline
						peak-=V_avg
						break
					EndIf
				EndFor
				
				break
			EndIf
			i+=1
			p+=1
		While(p<p+20000)
		
	Else
		print "No BSA Peak was found! Point:",pb
		print Baseline, Baseline+100, V_flag
		print nameofwave(w),w[p]
	EndIf
	BSATemp[0]= peak
	return BSATemp
End